@@ -6,7 +6,7 @@ use Time::HiRes qw(sleep);
 use JSON::PP ();
 use Scalar::Util qw(looks_like_number);
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 my %COLLECTORS = (
     system  => \&_system_info,
@@ -15,6 +15,7 @@ my %COLLECTORS = (
     mem     => \&_memory_usage,
     disk    => \&_disk_usage,
     disk_io => \&_disk_io,
+    mounts  => \&_mount_info,
     net     => \&_network_io,
     process => \&_process_list,
 );
@@ -28,7 +29,7 @@ sub collect {
     my ($which, $options) = @_;
     my @names;
     if (!defined $which) {
-        @names = qw(system cpu load mem disk disk_io net);
+        @names = qw(system cpu load mem disk disk_io mounts net);
         if ($options && ref $options eq 'HASH' && exists $options->{process}) {
             push @names, 'process';
         }
@@ -127,47 +128,57 @@ sub _memory_usage {
 }
 
 sub _disk_usage {
+    my $mounts = _mounted_filesystems();
     my %seen;
     my @disks;
     my $has_statvfs = POSIX->can('statvfs');
     my $df_stats    = _df_stats();
 
-    if (open my $fh, '<', '/proc/mounts') {
-        while (my $line = <$fh>) {
-            my ($device, $mount, $type) = (split /\s+/, $line)[0..2];
-            next if $seen{$mount}++;
-            next if $mount =~ m{^/(?:proc|sys|dev|run|snap)};
-            next if $type =~ /^(?:proc|sysfs|tmpfs|devtmpfs|cgroup.+|rpc_pipefs|overlay)$/;
-            next unless defined $mount && length $mount;
-            next unless -d $mount;
+    for my $mount_info (@$mounts) {
+        my $mount = $mount_info->{mount};
+        next if $seen{$mount}++;
+        next unless -d $mount;
 
-            my ($total, $used, $free);
-            if ($has_statvfs) {
-                my @stat = eval { POSIX::statvfs($mount) };
-                next unless @stat;
-                my ($bsize, $frsize, $blocks, $bfree, $bavail) = @stat;
-                $total = $blocks * $frsize;
-                $free  = $bavail * $frsize;
-                $used  = $total - ($bfree * $frsize);
-            } else {
-                my $info = $df_stats->{$mount};
-                next unless $info;
+        my ($total, $used, $free, $inode_total, $inode_used, $inode_free);
+        if ($has_statvfs) {
+            my @stat = eval { POSIX::statvfs($mount) };
+            if (@stat) {
+                my ($bsize, $frsize, $blocks, $bfree, $bavail, $files, $ffree) = @stat;
+                $total       = $blocks * $frsize;
+                $free        = $bavail * $frsize;
+                $used        = $total - ($bfree * $frsize);
+                $inode_total = _maybe_number($files);
+                $inode_free  = _maybe_number($ffree);
+                $inode_used  = defined $inode_total && defined $inode_free
+                    ? $inode_total - $inode_free
+                    : undef;
+            }
+        }
+
+        if (!defined $total) {
+            my $info = $df_stats->{$mount};
+            if ($info) {
                 $total = $info->{total};
                 $used  = $info->{used};
                 $free  = $info->{avail};
             }
-
-            push @disks, {
-                mount        => $mount,
-                filesystem   => $device,
-                type         => $type,
-                total_bytes  => $total,
-                used_bytes   => $used,
-                free_bytes   => $free,
-                used_pct     => _percent($used, $total),
-            };
         }
-        close $fh;
+
+        push @disks, {
+            mount           => $mount,
+            filesystem      => $mount_info->{device},
+            type            => $mount_info->{type},
+            options         => $mount_info->{options},
+            read_only       => $mount_info->{read_only},
+            total_bytes     => $total,
+            used_bytes      => $used,
+            free_bytes      => $free,
+            used_pct        => _percent($used, $total),
+            inodes_total    => $inode_total,
+            inodes_used     => $inode_used,
+            inodes_free     => $inode_free,
+            inodes_used_pct => _percent($inode_used, $inode_total),
+        };
     }
 
     if (!@disks && $df_stats && %$df_stats) {
@@ -179,10 +190,16 @@ sub _disk_usage {
                 mount        => $mount,
                 filesystem   => $info->{filesystem},
                 type         => $info->{type} // 'unknown',
+                options      => [],
+                read_only    => undef,
                 total_bytes  => $info->{total},
                 used_bytes   => $info->{used},
                 free_bytes   => $info->{avail},
                 used_pct     => _percent($info->{used}, $info->{total}),
+                inodes_total    => undef,
+                inodes_used     => undef,
+                inodes_free     => undef,
+                inodes_used_pct => undef,
             };
         }
     }
@@ -190,6 +207,26 @@ sub _disk_usage {
     return \@disks;
 }
 
+sub _mount_info {
+    my $mounts = _mounted_filesystems();
+    my %by_mount;
+
+    for my $info (@$mounts) {
+        my $key = _mount_key($info->{mount});
+        next unless length $key;
+        $by_mount{$key} = {
+            present     => 1,
+            mount       => $info->{mount},
+            filesystem  => $info->{device},
+            type        => $info->{type},
+            options     => $info->{options},
+            read_only   => $info->{read_only},
+        };
+    }
+
+    return \%by_mount;
+}
+
 sub _disk_io {
     open my $fh, '<', '/proc/diskstats' or return [];
     my @devices;
@@ -420,6 +457,46 @@ sub _df_stats {
     return \%stats;
 }
 
+sub _mounted_filesystems {
+    my %seen;
+    my @mounts;
+
+    if (open my $fh, '<', '/proc/mounts') {
+        while (my $line = <$fh>) {
+            my ($device, $mount, $type, $opts) = (split /\s+/, $line)[0..3];
+            next if $seen{$mount}++;
+            next if $mount =~ m{^/(?:proc|sys|dev|run|snap)};
+            next if $type =~ /^(?:proc|sysfs|tmpfs|devtmpfs|cgroup.+|rpc_pipefs|overlay)$/;
+            next unless defined $mount && length $mount;
+
+            my @options = defined $opts && length $opts ? split(/,/, $opts) : ();
+            my $read_only = grep { $_ eq 'ro' } @options ? 1 : 0;
+
+            push @mounts, {
+                device    => $device,
+                mount     => $mount,
+                type      => $type,
+                options   => \@options,
+                read_only => $read_only,
+            };
+        }
+        close $fh;
+    }
+
+    return \@mounts;
+}
+
+sub _mount_key {
+    my ($path) = @_;
+    return '' unless defined $path && length $path;
+    my $key = $path;
+    $key =~ s{^/}{root_};
+    $key =~ s{[^A-Za-z0-9_.]}{_}g;
+    $key =~ s{_+}{_}g;
+    $key =~ s{_\z}{};
+    return $key;
+}
+
 sub to_json {
     my ($data, %opts) = @_;
     my $encoder = JSON::PP->new->canonical->ascii(0);
